
    /////////////////////////////////////////////////
    // WeIRCd LINKING MODULE for WLINKSv1 Protocol //
    /////////////////////////////////////////////////
/*
  This module is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
  This module is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  The WeIRCd main folder should contain a copy of the GNU General Public
  License v3. If not, see <http://www.gnu.org/licenses/>.*/



//////// HEADER BLOCK //////////////
// Some internal definitions for
// this module and function headers


//
//  IMPORTANT!! Global vars are > BAD < !!!
//  =======================================
//
//   Different links of the same protocol will _share_
//   the values of global vars and OVERWRITE them respectively!
//   Please use wlinks_getStr() wlinks_setStr() wlinks_getInt() and wlinks_setInt() to
//   save data associated with _this_ link which is owned by it and not overwritten by
//   others
//   Do _NEVER_ use global vars in a WeIRCd linking module!

#define SVIDCOUNT 50 //count of server ids assigned to server names
#define OS SVIDCOUNT //offset of free data blocks (which aren't used for server ids->names)
#define PROTVERSION 39 //protocol version/build
void wlinksv1_syncservers();
void wlinksv1_syncusers();
void wlinksv1_syncchannels();
void wlinksv1_setName(const unsigned char* name);
const unsigned char* wlinksv1_getName();
void wlinksv1_setOtherName(const unsigned char* name);
const unsigned char* wlinksv1_getOtherName();
void wlinksv1_channelmsg(const unsigned char* chan, const unsigned char* content);
void wlinksv1_resyncChannel(const unsigned char* chan);


//////// INIT BLOCK //////////////
// Set some link specific settings
// and send login code for active
// links

void wlinksv1_init() {
	if (wlinks_setLineBufferSize(1500) < 0) {
	//every line in our protocol can be up to 699 chars long (including line break)
		return;//quit on fail
	}
	if (currentLinkType == LINKTYPE_ACTIVE) {
	//it's an active link, so send login as we are connected now
		sl("wlinksv1 :%s",wlinks_info_getShortPass());
		return;
	}else{
	//no active link - DO NOTHING!! this is important and absolutely needed
		return;
	}
}



//////// PROCESSING BLOCK ////////////
// Process the stuff weircd receives
// through the link connection

unsigned char origdata[1000]; //this global var is ok, as it is a temporary buffer that can be used by multiple module instances at once without any problems
void wlinksv1_receive(unsigned char* data) {
	strncpy(origdata,data,999);origdata[999] = 0;
	unsigned char tmp[500];
	unsigned char tmp2[100];
	//LOGIN PROCESS BLOCK:
	if (wlinks_getInt(OS+0) == 0) {//link not established
		if (currentLinkType == LINKTYPE_PASSIVE) {
			if (strcasecmp(wlinks_getStr(OS+0),"") == 0) {
				if (strstw(data,"wlinksv1 :") != true) {
					wlinks_disconnect();
				}else{
					cut(data,strlen("wlinksv1 :"));
					if (strcasecmp(data,wlinks_info_getShortPass()) != 0) {
						wlinks_disconnect();
						return;
					}
					snprintf(tmp,49,"%ld",ts()+vrandom(-50,50));
					tmp[49] = 0;
					sl("hash :%s",wlinks_md5(tmp));//save hash and send it
					wlinks_setStr(OS+0,wlinks_md5(tmp));
				}
			}else{
				//check the received hash which should be md5(our send hash+linking password)
				if (strstw(data,"login :") != true) {wlinks_disconnect();return;}
				cut(data,strlen("login :"));
				if (strlen(data) != 32) {wlinks_disconnect();return;} //no md5 hash
				snprintf(tmp,49,"%s%s",wlinks_getStr(OS+0),wlinks_info_getPass());tmp[49] = 0;
				if (strcasecmp(data,wlinks_md5(tmp)) != 0) {
					wlinks_disconnect(); //wrong hash result
				}else{
					wlinks_established();
					sl("protocolversion :%d",PROTVERSION);
					wlinks_setInt(OS+0,1);
					return;
				}
			}
		}else{
			if (wlinks_getInt(OS+1) == 0) {
				if (strstw(data,"hash :") != true) {
					wlinks_disconnect();
				}else{
					cut(data,strlen("hash :"));
					if (strlen(data) != 32) {wlinks_disconnect();return;} //no md5 hash
					snprintf(tmp,49,"%s%s",data,wlinks_info_getPass());tmp[49] = 0;//generate a new hash of the received hash an the linking password
					sl("login :%s",wlinks_md5(tmp));//send the generation back
					wlinks_setInt(OS+1,1);
				}
			}else{
				if (strstw(data,"protocolversion :") != true) {
					wlinks_disconnect();
				}else{
					cut(data,strlen("protocolversion :"));
					if (atoi(data) > PROTVERSION) {//newer than me
						wlinks_msg("Own protocol version too old! Needs updating before connecting to %s.",wlinks_info_target());
						wlinks_disconnect();
						return;
					}
					if (atoi(data) < PROTVERSION) {//older than me
						wlinks_msg("%s's protocol version too old! Cannot link to this.",wlinks_info_target());
						wlinks_disconnect();
						return;
					}
					wlinks_setInt(OS+0,1);
					wlinks_established();
					wlinksv1_setName(wlinks_info_ownName());//save the server name we sent! because if it changes locally, we still need to use the same name regarding linking stuff
					if (wlinks_info_needFullInfo() == true) {
						sl("link %s %lld services :%s",wlinksv1_getName(),ts(),wlinks_info_getDescription()); //register as services
					}else{
						sl("link %s %lld normal :%s",wlinksv1_getName(),ts(),wlinks_info_getDescription()); //register as normal server
					}
				}
			}
		}
	}else{
		if (wlinks_getInt(OS+0) == 1) {
			if (currentLinkType == LINKTYPE_PASSIVE) {
				unsigned char vtime[50];
				if (strstw(data,"link ") != true) {
					wlinks_disconnect();
					return;
				}
				cut(data,strlen("link "));
				if (charpos(data,' ') <= 0 || strlen(data) < 2) {
					wlinks_disconnect();return;
				}
				strncpy(tmp,data,99);tmp[99] = 0;
				if (charpos(tmp,' ') > 0) {tmp[charpos(tmp,' ')] = 0;}//tmp = server name
				cut(data,charpos(data,' ')+1);
				if (charpos(tmp,':') > 0) {return;}
				
				if (charpos(data,' ') <= 0) {return;}
				strncpy(vtime,data,59);vtime[59] = 0;cutonfirstspace(vtime);//actor = time stamp
				cut(data,charpos(data,' ')+1);trimleft(data);
				
				if (charpos(data,':') <= 0) {return;}
				strncpy(tmp2,data,99);tmp2[99] = 0;
				if (charpos(tmp2,':') > 0) {tmp2[charpos(tmp2,':')] = 0;trimright(tmp2);}//tmp2 = services or not
				cut(data,charpos(data,':')+1);trimleft(data);trimright(data);//data = server desc
				if (strcasecmp(tmp2,"services") != 0 && strcasecmp(tmp2,"normal") != 0) {wlinks_disconnect();return;}
				long long t = ts();
				t -= atoll(vtime);
				if (t < 0) {t = -t;}
				if (t > 15) {
					wlinks_msg("Server clocks are async (delay of %d seconds exceeds maximum of 15)!!",t);
					wlinks_disconnect();return;
				}
				if (wlinks_addServ(tmp,1,NULL,data) < 0) {
					wlinks_msg("Failed to register %s as %s!",wlinks_info_target(),tmp);
					wlinks_disconnect();return;
				}
				wlinksv1_setOtherName(tmp);
				wlinksv1_setName(wlinks_info_ownName());
				if (wlinks_info_needFullInfo() == true) {
					sl("now linked to %s services :%s",wlinksv1_getName(),wlinks_info_getDescription());
				}else{
					sl("now linked to %s normal :%s",wlinksv1_getName(),wlinks_info_getDescription());
				}
				wlinks_setInt(OS+0,2);
				wlinksv1_syncservers();
				wlinksv1_syncusers();
				wlinksv1_syncchannels();
				return;
			}else{
				if (strstw(data,"now linked to ") != true) {
					wlinks_disconnect();return;
				}
				cut(data,strlen("now linked to "));
				if (charpos(data,' ') <= 0 || strlen(data) < 2) {
					wlinks_disconnect();
					return;
				}
				
				strncpy(tmp,data,99);tmp[99] = 0;
				if (charpos(tmp,' ') > 0) {tmp[charpos(tmp,' ')] = 0;}//tmp = server name
				cut(data,charpos(data,' ')+1);
				if (charpos(tmp,':') > 0) {return;}
				
				if (charpos(data,':') < 0) {return;}
				strncpy(tmp2,data,99);tmp2[99] = 0;
				if (charpos(tmp2,':') > 0) {tmp2[charpos(tmp2,':')] = 0;trimright(tmp2);}//tmp2 = services or not
				cut(data,charpos(data,':')+1);trimleft(data);trimright(data);//data = server desc
				
				if (strcasecmp(tmp2,"services") != 0 && strcasecmp(tmp2,"normal") != 0) {wlinks_disconnect();return;}
				if (wlinks_addServ(tmp,1,NULL,data) < 0) {
					wlinks_msg("Failed to register %s as %s!",wlinks_info_target(),data);
					wlinks_disconnect();return;
				}
				wlinksv1_setOtherName(tmp);
				wlinks_setInt(OS+0,2);
				wlinksv1_syncservers();
				wlinksv1_syncusers();
				wlinksv1_syncchannels();
				return;
			}
		}
	}
	if (wlinks_getInt(OS+0) != 2) {return;}
	//NORMAL COMMAND PROCESSING
	if (strstw(data,"adduser :") == true) {
		cut(data,strlen("adduser :"));trimright(data);
		if (charpos(data,' ') <= 0) {return;}
		strncpy(tmp,data,99);tmp[99] = 0;
		cutonfirstspace(tmp);//nickname
		cut(data,charpos(data,' ')+1);//timestamp
		if (charpos(data,' ') > 0) {//ok, we got a specified server
			strncpy(tmp2,data,99);tmp2[99] = 0;
			cutonfirstspace(data);
			cut(tmp2,charpos(tmp2,' ')+1);
		}else{
			strcpy(tmp2,"");
		}
		long long stamp = atoll(data);
		if (stamp < 0) {return;}
		if (strcasecmp(tmp2,"") == 0) {//user is local for the linked server
			strcpy(tmp2,wlinksv1_getOtherName());//use our saved server name
		}else{//user is nonlocal for the linked server (=indirectly linked)
			if (wlinks_servThroughThisLink(tmp2) == false) {return;} //server not known o.O
		}
		if (wlinks_collideUser(tmp,stamp) == 0) {
			wlinks_addUser(tmp,tmp2);
			if (strcasecmp(tmp2,"") == 0) {//append other server's name as for us, it's no longer local
				trimright(origdata);
				strncat(origdata,wlinksv1_getOtherName(),999);
				origdata[999] = 0;
			}
			wlinks_forward(origdata);
		}else{
			sl(":%s KILL %s :Nick collision",wlinksv1_getName(),tmp);
		}
		return;
	}
	if (strstw(data,"addchan :") == true) {
		cut(data,strlen("addchan :"));trimright(data);
		if (charpos(data,' ') <= 0) {return;}
		strncpy(tmp,data,99);tmp[99] = 0;
		cutonfirstspace(tmp);//channame
		cut(data,charpos(data,' ')+1);//timestamp
		if (charpos(data,' ') > 0) {//ok, we got a specified server
			strncpy(tmp2,data,99);tmp2[99] = 0;
			cutonfirstspace(data);
			cut(tmp2,charpos(tmp2,' ')+1);
		}else{
			strcpy(tmp2,"");
		}
		long long stamp = atoll(data);
		if (stamp < 0) {return;}
		if (strcasecmp(tmp2,"") == 0) {//chan is local for the linked server
			strcpy(tmp2,wlinksv1_getOtherName());//use our saved server name
		}else{//channel is nonlocal for the linked server (=indirectly linked)
			if (wlinks_servThroughThisLink(tmp2) == false) {return;} //server not known o.O
		}
		if (wlinks_collideChannel(tmp,stamp) == 0) {
			wlinks_addChannel(tmp,tmp2);
			if (strcasecmp(tmp2,"") == 0) {//append other server's name as for us, it's no longer local
				trimright(origdata);
				strncat(origdata,wlinksv1_getOtherName(),999);
				origdata[999] = 0;
			}
			wlinks_forward(origdata);
		}else{
			sl(":%s DEOP %s",wlinksv1_getName(),tmp);
		}
		return;
	}
	if (strstw(data,"PONG :") == true) {
		wlinks_pong();
		return;
	}
	if (strstw(data,"PING :") == true) {
		cut(data,strlen("PING :"));trimleft(data);
		sl("PONG :%s",data);
		return;
	}
	if (strstw(data,"addserver ") == true) {
		cut(data,strlen("addserver "));trimright(data);
		if (charpos(data,' ') <= 0) {wlinks_disconnect();return;}
		strncpy(tmp,data,99);tmp[99] = 0;
		cutonfirstspace(tmp);//servername
		cut(data,charpos(data,' ')+1);//hopcount
		unsigned char desc[200];
		if (charpos(data,' ') > 0 && charpos(data,':') > charpos(data,' ')+1) {//ok, we got a specified server
			strncpy(tmp2,data,99);tmp2[99] = 0;
			cutonfirstspace(data);
			cut(tmp2,charpos(tmp2,' ')+1);
			if (charpos(tmp2,':') < 0 || charpos(tmp2,':') > 30) {wlinks_disconnect();return;}//cut off the added desc
			strncpy(desc,tmp2,149);desc[149] = 0;cut(desc,charpos(desc,':')+1);trimleft(desc);trimright(desc);
			tmp2[charpos(tmp2,':')] = 0;trimright(tmp2);
		}else{
			strcpy(tmp2,"");
			if (charpos(data,':') < 0 || charpos(data,':') > 30) {wlinks_disconnect();return;}//cut off the added desc
			strncpy(desc,data,149);desc[149] = 0;cut(desc,charpos(desc,':')+1);trimleft(desc);trimright(desc);
			data[charpos(data,':')] = 0;trimright(data);
		}
		if (strlen(desc) <= 1) {return;}
		int hopcount = atoi(data);
		if (hopcount < 2) {return;}
		if (strcasecmp(tmp2,"") == 0) {//user is local for the linked server
			strcpy(tmp2,wlinksv1_getOtherName());//use our saved server name
		}
		if (wlinks_addServ(tmp,hopcount,tmp2,desc) < 0) {
			wlinks_msg("Failed to add %s!",data);
			wlinks_disconnect();return; //might just be an accidential double event (can happen), but we still need to disconnect to be sure
		}
		return;
	}
	/*if (strstw(data,"touser ") == true) {
		cut(data,strlen("touser "));
		unsigned char actor[100];
		strncpy(actor,data,99);actor[99] = 0;
		if (charpos(actor,' ') <= 0) {return;}
		cut(data,charpos(data,' ')+1);cutonfirstspace(actor);
		wlinks_sndMsgToUser(actor,data);
		return;
	}*/
	if (strstw(data,"delserver ") == true) {
		cut(data,strlen("delserver "));
		while (charpos(data,' ') > 0) {
			strncpy(tmp2,data,99);tmp2[99] = 0;
			if (charpos(tmp2,' ') <= 0) {wlinks_disconnect();return;}//malformed
			tmp2[charpos(tmp2,' ')] = 0;
			cut(data,charpos(data,' ')+1);
			wlinks_delServ(tmp2);
		}
		if (strlen(data) > 0) {
			wlinks_delServ(data);
		}
		return;
	}
	if (strstw(data,"delchannel ") == true) {
		cut(data,strlen("delchannel "));
		while (charpos(data,' ') > 0) {
			strncpy(tmp2,data,99);tmp2[99] = 0;
			if (charpos(tmp2,' ') <= 0) {wlinks_disconnect();return;}//malformed
			tmp2[charpos(tmp2,' ')] = 0;
			cut(data,charpos(data,' ')+1);
			wlinks_delChan(tmp2);
		}
		if (strlen(data) > 0) {
			wlinks_delChan(data);
		}
		return;
	}
	if (strstw(data,"resyncchan :") == true) {
		cut(data,strlen("resyncchan :"));
		int r = wlinks_channel_getHandle(data);
		if (r < 0) {return;}
		wlinksv1_resyncChannel(data);
		wlinks_forward(origdata);
		return;
	}
	if (strstw(data,":") == true) {//it's some kind of event
		cut(data,1);
		unsigned char actor[100];
		strncpy(actor,data,99);actor[99] = 0;
		if (charpos(data,' ') > 0) {
			if (charpos(actor,' ') > 0) {
				actor[charpos(actor,' ')] = 0;
			}
			cut(data,charpos(data,' ')+1);trimleft(data);
		}else{return;}
		if (strstw(actor,"!") == true) {return;}//broken event, do not forward
		unsigned char event[50];
		strncpy(event,data,49);event[49] = 0;
		if (charpos(data,' ') > 0) {
			if (charpos(event,' ') > 0) {
				event[charpos(event,' ')] = 0;
			}
			cut(data,charpos(data,' ')+1);trimleft(data);
		}else{return;}
		if (isnumber(event) == true) {//numeric, forward it!
			trimleft(data);cutonfirstspace(data);
			if (strlen(data) > 0) {
				int r = wlinks_user_getHandle(data);
				if (r >= 0) {
					wlinks_sndMsgToUser(data,origdata);
				}
			}
			return;
		}
		if (strcasecmp(event,"NICK") == 0) {//someone renames
			if (strstw(data,":") == true) {cut(data,1);trimleft(data);}
			if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
			int r = wlinks_user_getHandle(actor);
			if (r >= 0) {
				wlinks_renameUser(actor,data);
			}
			wlinks_forward(origdata);//forward to all other links
			return;
		}
		if (strcasecmp(event,"QUIT") == 0) {//someone quits
			if (strstw(data,":") == true) {cut(data,1);trimleft(data);}
			if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
			int r = wlinks_user_getHandle(actor);
			if (r >= 0) {
				wlinks_removeUser(actor,data);
			}
			wlinks_forward(origdata);//forward to all other links
			return;
		}
		if (strcasecmp(event,"PART") == 0) {//someone parts a channel
			if (charpos(data,':') > 0) {//extract reason
				strncpy(tmp,data,499);tmp[499] = 0;
				if (charpos(tmp,':') > 0) {
					cut(tmp,charpos(tmp,':')+1);trimleft(tmp);
				}else{
					strcpy(tmp,"");
				}
				data[charpos(data,':')] = 0;trimright(data);
			}
			if (strlen(tmp) < 1) {//if no reason specified, use the user's nick
				strncpy(tmp,actor,499);tmp[499] = 0;
				if (charpos(tmp,'!') > 0) {tmp[charpos(tmp,'!')] = 0;}
			}
			int u = wlinks_user_getHandle(actor);
			if (u < 0) {return;}
			int k = wlinks_channel_getHandle(data);
			if (k < 0) {return;}
			if (wlinks_user_getServer(u) == NULL || wlinks_channel_getServer(k) != NULL) {
				wlinks_partUser(data,actor,tmp);
			}
			wlinks_forward(origdata);//forward to all other links
			return;
		}
		if (strcasecmp(event,"JOIN") == 0) {//someone joins a channel
			if (strstw(data,":") == true) {cut(data,1);trimleft(data);}
			sprintf(tmp,actor,99);tmp[99] = 0;
			if (charpos(tmp,'!') > 0) {tmp[charpos(tmp,'!')] = 0;}
			printf("joining of %s into '%s'\n",actor,data);
			int u = wlinks_user_getHandle(tmp);
			if (u < 0) {return;}
			int k = wlinks_channel_getHandle(data);
			if (k < 0) {return;}
			if (wlinks_user_getServer(u) == NULL || wlinks_channel_getServer(k) == NULL) {
				//it's either my channel or my user
				wlinks_joinUser(data,actor);
			}
			wlinks_forward(origdata);
			return;
		}
		if (strcasecmp(event,"SILENTJOIN") == 0) {//silent join for resyncing
			if (strstw(data,":") == true) {cut(data,1);trimleft(data);}
			sprintf(tmp,actor,99);tmp[99] = 0;
			if (charpos(tmp,'!') > 0) {tmp[charpos(tmp,'!')] = 0;}
			//printf("silent joining of %s into '%s'\n",actor,data);
			int u = wlinks_user_getHandle(tmp);
			if (u < 0) {return;}
			int k = wlinks_channel_getHandle(data);
			if (k < 0) {return;}
			if (wlinks_user_getServer(u) == NULL || wlinks_channel_getServer(k) == NULL) {
				//it's either my channel or my user
				wlinks_joinUserSilently(data,actor);
			}
			wlinks_forward(origdata);
			return;
		}
		if (strcasecmp(event,"WANTJOIN") == 0) {//someone WANTS joins a channel
			printf("it's wantjoin!");
			if (strstw(data,":") == true) {cut(data,1);trimleft(data);}
			strncpy(tmp,data,99);tmp[99] = 0;
			if (charpos(tmp,' ') > 0) {//we got a channel password!
				cut(tmp,charpos(tmp,' ')+1);
			}else{
				strcpy(tmp,"");
			}
			printf("wantjoin: %s - %s\n",tmp,data);
			cutonfirstspace(data);
			int k = wlinks_channel_getHandle(data);
			if (k < 0) {printf("channel not found!\n");return;}
			printf("channel found...\n");
			if (wlinks_channel_getServer(k) == NULL) {
				//it's one of my channels
				if (strcasecmp(tmp,"") != 0) {
					wlinks_processJoin(data,actor,tmp);//password
				}else{
					printf("join without password\n");
					wlinks_processJoin(data,actor,NULL);//no password
				}
			}else{
				//not my channel, forward it to the channel owning server
				wlinks_sndMsgToServer(wlinks_channel_getServer(k),origdata);
			}
			return;
		}
		if (strcasecmp(event,"TOPIC") == 0) {//topic cmd
			if (charpos(data,' ') > 0) {//topic change, not only request
				strncpy(tmp,data,499);tmp[499] = 0;
				if (charpos(tmp,' ') > 0) {
					cut(tmp,charpos(tmp,' ')+1);trimleft(tmp);
				}else{
					strcpy(tmp,"");
				}
			}
			if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
			int j = wlinks_user_getHandle(actor);
			if (j < 0) {return;}
			cutonfirstspace(data);
			int k = wlinks_channel_getHandle(data);
			if (k < 0) {return;}
			if (wlinks_channel_getServer(k) == NULL) {
				//it's one of my channels
				if (strcasecmp(tmp,"") != 0) {
					//topic change - TODO: implement this!
					
				}else{
					wlinks_sendTopic(actor,data);
				}
			}else{
				//not my channel, forward it to the channel owning server
				wlinks_sndMsgToServer(wlinks_channel_getServer(k),origdata);
			}
			return;
		}
		if (strcasecmp(event,"NAMES") == 0) {//someone executes names on a chan
			if (strstw(data,":") == true) {cut(data,1);trimleft(data);}
			if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
			int r = wlinks_channel_getHandle(data);
			int j = wlinks_user_getHandle(actor);
			if (j < 0 || r < 0) {return;}
			if (wlinks_channel_getServer(r) == NULL) {
				//the chan is local
				wlinks_sendNamesToUser(data,actor);
			}else{
				//the chan is nonlocal, forward
				wlinks_sndMsgToServer(wlinks_channel_getServer(r),origdata);
			}
			return;
		}
		if (strcasecmp(event,"WHOIS") == 0) {//someone whois's
			if (strstw(data,":") == true) {cut(data,1);trimleft(data);}
			if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
			int r = wlinks_user_getHandle(data);
			int j = wlinks_user_getHandle(actor);
			if (j < 0) {return;}
			if (r >= 0) {
				if (wlinks_user_getServer(r) == NULL) {
				//answer the whois
					wlinks_user_sendWhois(j,r);
				}else{
				//forward it
					wlinks_sndMsgToUser(data,origdata);
				}
			}else{
				if (wlinks_info_isServicesBot(data) == true) {
					wlinks_user_sendServicesWhois(j,data);
				}else{
					sl(":%s 401 %s %s :No such nick",wlinksv1_getName(),wlinks_user_getNickname(j),data);
				}
			}
			return;
		}
		if (strcasecmp(event,"MOTD") == 0) {//someone motd's
			if (strstw(data,":") == true) {cut(data,1);trimleft(data);}
			if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
			printf("%s wants motd from %s (%s)\n",actor,data,wlinksv1_getName());
			int j = wlinks_user_getHandle(actor);
			if (j < 0) {return;}
			if (strcasecmp(data,wlinksv1_getName()) == 0) {
				motd_send(j);//answer mot request
			}else{
				wlinks_sndMsgToServer(data,origdata);//forward whois request
			}
			return;
		}
		if (strcasecmp(event,"PRIVMSG") == 0 || strcasecmp(event,"NOTICE") == 0) {//someone sends something
			if (charpos(data,' ') <= 0) {return;}//parameter missing
			strncpy(tmp,data,59);tmp[59] = 0;
			if (charpos(tmp,' ') > 0) {tmp[charpos(tmp,' ')] = 0;}
			cut(data,charpos(data,' ')+1);
			if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
			
			if (tmp[0] == '#') {
				wlinks_channelMsg(tmp,origdata);
				wlinks_forward(origdata);
			}else{
				int r = wlinks_user_getHandle(tmp);
				if (r >= 0) {
					wlinks_sndMsgToUser(tmp,origdata);//forward privmsg to user
				}else{
					sl(":%s 401 %s %s :No such nick",wlinksv1_getName(),actor,data);
				}
			}
			return;
		}
		if (strcasecmp(event,"KILL") == 0) {//someone gets killed
			if (charpos(data,' ') <= 0) {return;}//parameter missing
			strncpy(tmp,data,59);tmp[59] = 0;
			if (charpos(tmp,' ') > 0) {tmp[charpos(tmp,' ')] = 0;}
			cut(data,charpos(data,' ')+1);
			int r = wlinks_user_getHandle(tmp);
			if (r < 0) {
				if (wlinks_info_isServicesBot(tmp) == true) {
					if (charpos(actor,'!') <= 0) {
					//the services got killed by a server => multiple services on the network?
						wlinks_disconnect();
					}
					return;
				}
				return;
			}
			wlinks_removeUser(tmp,data);
			return;
		}
	}
}




//////// SEND BLOCK ////////////
// Process the stuff weircd wants
// to send (we need to alter the server
// name if it changed and can do some
// other tests here) - "data" contains
// one line which is supposed to reach
// the other server or one of those
// indirectly connected through this link
// and is an IRC event, e.g.
// :LocalUserHere!blu@bla PRIVMSG otheruseronthislink :query-msg that needs to get transmitted there

void wlinksv1_send(const unsigned char* data) {
	unsigned char tmp[600];
	unsigned char tmp2[600];
	unsigned char tmp3[600];
	printf("-%s\n",data);
	strncpy(tmp2,data,599);tmp2[599] = 0;
	strcpy(tmp,tmp2);
	if (strstw(tmp,":") == true) {
		if (charpos(tmp,' ') < charpos(tmp,'!') && charpos(tmp,' ') > 0) {//it seems to be an event from the server
			// => set our saved server name which can be different in case the "real" one changed
			cut(tmp,charpos(tmp,' ')+1);
			snprintf(tmp2,599,":%s %s",wlinksv1_getName(),tmp);tmp2[599] = 0;
		}
	}
	strcpy(tmp,tmp2);
	//check for channel privmsg
	if (strstw(tmp,":") == true && charpos(tmp,' ') > 0) {
		cut(tmp,charpos(tmp,' ')+1);
		strcpy(tmp3,tmp);
		cutonfirstspace(tmp);
		//if (charpos(tmp3,':') > 0) {tmp3[charpos(tmp3,':')] = 0;trimright(tmp3);}
		if (strcasecmp(tmp,"PRIVMSG") == 0) {
			if (tmp3[0] == '#') {//to a channel!
				strcpy(tmp2,tmp3);
				if (charpos(tmp3,':') <= 0) {return;}
				cut(tmp3,charpos(tmp3,':')+1);
				tmp2[charpos(tmp2,':')] = 0;trimright(tmp2);
				wlinksv1_channelmsg(tmp2,tmp3);
				return;
			}
		}
	}
	wlinks_sendLine(tmp2);
}





//////// ABSTRACTION BLOCK ////////////
// Various abstracting functions
// covering usage we don't want to
// have directly in some of the
// functions above

void wlinksv1_syncservers() {
	wlinks_cycleServers_start();
	int r = wlinks_cycleServers_next();
	while (r >= 0) {
		if (wlinks_cycleServers_getLinkTarget(r) != NULL) {
			sl("addserver %s %d %s :%s",wlinks_cycleServers_getName(r),wlinks_cycleServers_hopcount(r)+1,wlinks_cycleServers_getLinkTarget(r),wlinks_cycleServers_getDesc(r));
		}else{
			sl("addserver %s %d :%s",wlinks_cycleServers_getName(r),2,wlinks_cycleServers_getDesc(r));
		}
		r = wlinks_cycleServers_next();
	}
}
void wlinksv1_syncusers() {
	//just send out info on all users we know about which are not received from this link itself
	wlinks_cycleUsers_start(USERCYCLE_ALLOTHERS);
	int r = wlinks_cycleUsers_next();
	while (r >= 0) {
		if (wlinks_user_getServer(r) != NULL) {
			sl("adduser :%s %lld %s",wlinks_user_getNickname(r),wlinks_user_getTS(r),wlinks_user_getServer(r));
		}else{
			sl("adduser :%s %lld",wlinks_user_getNickname(r),wlinks_user_getTS(r));
		}
		r = wlinks_cycleUsers_next();
	}
	if (wlinks_info_needFullInfo() == true) {
		sl("adduser :NickServ 0");
		sl("adduser :ChanServ 0");
		sl("adduser :Global 0");
		sl("adduser :OperServ 0");
	}
}
void wlinksv1_syncchannels() {
	//just send out info on all channels we know about which are not received from this link itself
	wlinks_cycleChannels_start(CHANNELCYCLE_ALLOTHERS);
	int r = wlinks_cycleChannels_next();
	while (r >= 0) {
		if (wlinks_channel_getServer(r) != NULL) {
			sl("addchan :%s %lld %s",wlinks_channel_getName(r),wlinks_channel_getTS(r),wlinks_channel_getServer(r));
		}else{
			sl("addchan :%s %lld",wlinks_channel_getName(r),wlinks_channel_getTS(r));
		}
		wlinks_introduceChannelUsers(r);
		r = wlinks_cycleUsers_next();
	}
}
void wlinksv1_resyncChannel(const unsigned char* chan) {
	int r = wlinks_channel_getHandle(chan);
	if (r < 0) {return;}
	wlinks_introduceChannelUsersSilently(r);
	r = wlinks_cycleUsers_next();
}
void wlinksv1_setName(const unsigned char* name) {
	wlinks_setStr(OS+0,name);
}
const unsigned char* wlinksv1_getName() {
	return wlinks_getStr(OS+0);
}
void wlinksv1_setOtherName(const unsigned char* name) {
	wlinks_setStr(OS+1,name);
}
const unsigned char* wlinksv1_getOtherName() {
	return wlinks_getStr(OS+1);
}
void wlinksv1_channelmsg(const unsigned char* chan, const unsigned char* content) {
	unsigned char send[580];
	int k = wlinks_channel_getHandle(chan);
	if (k < 0) {return;}
	wlinks_cycleUsers_start(USERCYCLE_ALL);
	int r = wlinks_cycleUsers_next();
	while (r >= 0) {
		if (wlinks_channelContainsUser(chan,wlinks_user_getNickname(r)) == true) {
			//the user is inside the channel
			snprintf(send,579,"touser %s %s",wlinks_user_getNickname(r),content);send[579] = 0;
			if (wlinks_user_getServer(r) == NULL) {
			//local user, pass the msg to the user
				wlinks_sndMsgToUser(wlinks_user_getNickname(r),content);
			}else{
			//nonlocal user
				wlinks_sndMsgToUser(wlinks_user_getNickname(r),send);
			}
		}
		r++;
	}
}


