//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007, 2008 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


// Those functions and definitions are available for a linker module

#define LINKTYPE_ACTIVE 1
#define LINKTYPE_PASSIVE 2
#define MAXLINKS 50

#define PLUGINDATA_MAXINT 250
#define PLUGINDATA_MAXSTR 70

#define wlinks_sendLine sl //you can use sl() as shortcut instead of wlinks_sendLine()

int currentLinkType; //check for LINKTYPE_PASSIVE or LINKTYPE_ACTIVE






////        SECTION A - info section

bool wlinks_info_isServicesBot(const unsigned char* nick);
  // is this nickname currently used by a built-in services bot?
const unsigned char* wlinks_info_ownName();
  // get the name of this server (e.g. irc.something.net) !IMPORTANT! this is subject to change without care about any links, so save a copy of this if you need it (using wlinks_setStr() in section c)
const unsigned char* wlinks_info_getPass();
  // get the current linking password
const unsigned char* wlinks_info_getDescription();
  // get the description of this server (specified by its admins)
const unsigned char* wlinks_info_getShortPass();
  // get the first 8 chars of the current linking password (or just the whole password in case it is exactly of 8 chars length or less)
void wlinks_msg(const unsigned char* line,...);
  // sends a [LINKS] notice to all opers, can be used to tell if a link has been successfully established etc
void wlinks_established();
  // tell weircd's link management that the link is now established (password login successful) and that your module is now starting to sync users etc
const unsigned char* wlinks_md5(const unsigned char* value);
  // create an md5 hash
const unsigned char* wlinks_info_target();
  // get the target hostname or ip
bool wlinks_info_needFullInfo();
  // returns true if this server is a services server and therefore needs to get information about every channel join and not only those affecting local channels






////        SECTION B - generic connection functions

int wlinks_setLineBufferSize(int len);
  // set the maximum receive and send line len - do this in the init function! cannot be changed later
void wlinks_sendLine(const unsigned char* line,...);
  // needs a nullterminated string and automatically adds \r\n to it
void wlinks_disconnect();
  // immediate closing of the link, also discards any data assigned to it (see section c) and automatically netsplits/removes all the affected users
void wlinks_forward(const unsigned char* line);
  // does the same as wlinks_sendLine, but doesn't send it to the currently active link, but puts it onto the IRC event queue of all other links (no matter which protocol) to spread an event (remember - it does NOT send it to the active link itself!)
void wlinks_sndMsgToServer(const unsigned char* name,const unsigned char* msg);
  // send an IRC event message to a specific server
void wlinks_pong();
  // tells WeIRCd that a pong to it's ping arrived






////        SECTION C - storing data (assigned to the link and still available on later access)

void wlinks_setInt(int index, int value); // index: 0 - PLUGINDATA_MAXINT
int wlinks_getInt(int index); // index: 0 - PLUGINDATA_MAXINT
void wlinks_setStr(int index, const unsigned char* value); // index: 0 - PLUGINDATA_MAXSTR; needs to be nullterminated string
const unsigned char* wlinks_getStr(int index); // index: 0 - PLUGINDATA_MAXSTR; will never return NULL but "" in case there is no string set yet






////        SECTION D - managing the servers known to WeIRCd through a link

int wlinks_addServ(const unsigned char* servname, int hop, const unsigned char* linktarget, const unsigned char* description);
  // introduce a new server and its hopcount (1 - directly linked, >1 indirectly linked), returns -1 on failure (server already known or similar). if the hopcount is 1, linktarget is myself = NULL, otherwise specify the server which it is linked to
void wlinks_delServ(const unsigned char* servname);
  // removes a server and automatically netsplits/removes all users assigned to it (you should not remove yourself using this function, use wlinks_disconnect() for this), best used for telling linked servers about other ones which were connected to you and now split up
bool wlinks_servLinked(const unsigned char* servername);
  // is this server linked in any way?
bool wlinks_servOtherLinked(const unsigned char* servername);
  // is this server linked but NOT through this link itself (which means we need to forward data)?
bool wlinks_servThroughThisLink(const unsigned char* servname);
  // is this server added/available through this link itself?
// ##### -- cycling through servers: --
void wlinks_cycleServers_start();
  // cycle through all other servers not known through this link itself
int wlinks_cycleServers_next();
  // get a server handle of the next server in the list or -1 if there is no other server left
const unsigned char* wlinks_cycleServers_getName(int handle);
  // get the name of the server with this cycle handle
const unsigned char* wlinks_cycleServers_getLinkTarget(int handle);
  // get the link target of the server or NULL if it's directly linked
const unsigned char* wlinks_cycleServers_getDesc(int handle);
  // get the link target of the server or NULL if it's directly linked
int wlinks_cycleServers_hopcount(int handle);
  // get the hop count or 0 if it's directly linked






////        SECTION E - users - get infos, add new ones or quit them

// ##### -- cycling through the known userbase: --
#define USERCYCLE_LOCAL 1
#define USERCYCLE_ALL 2
#define USERCYCLE_ALLOTHERS 3
//IMPORTANT! After returning control to WeIRCd and leaving the module code after processing something, the cycles are reset until the module gains control next time
void wlinks_cycleUsers_start(int cycletype);
  // start a new loop through all the users matching the criteria specified by the type
int wlinks_cycleUsers_next();
  // get the handle for the next user in the currently running user cycling
void wlinks_cycleUsers_reset();
  // reset the user cycling (not necessary in normal cases, just use wlinks_cycleUsers_start(type) to start a new one and it will automatically stop the old correctly)

// ##### -- get information on a user handle --
//IMPORTANT! After returning control to WeIRCd and leaving the module code after processing something, all handles become UNDEFINED and invalid!! If you want to remember some user for a longer time, use his/her nickname!!
int wlinks_user_getHandle(const unsigned char* nick);
  // returns a user handle for the user with this nickname or -1 when no such nick is in use (it still might be used by local built-in services though!)
const unsigned char* wlinks_user_getServer(int handle);
  // returns the server this user is on. in case it is the local server, this returns NULL!
bool wlinks_user_throughThisLink(int handle);
  // returns true if this user is on a sever introduced and connected through this link itself
const unsigned char* wlinks_user_getNickname(int handle);
  // returns the nickname of this user. expect this to return never NULL if your handle is valid
long long wlinks_user_getTS(int handle);
  // returns the timestamp when the user did the last rename
void wlinks_user_sendWhois(int to, int about);
  // sends whois information about one user to another one, putting it automatically onto the send queue of the responsible link
void wlinks_user_sendServicesWhois(int to, const unsigned char* bot);
  // sends whois information for a services bot of the built-in services

// ##### -- add and remove users
int wlinks_collideUser(const unsigned char* nickname, long long timestamp);
  // this function is going to quit any users with this nickname when their nick rename timestamp is newer than the specified one and return 0 (also if there are no users with this nick). If there is a user which renamed earlier than this timestamp, then the function returns -1 and keeps that user.
void wlinks_addUser(const unsigned char* nickname, const unsigned char* servername);
  // add a specific user from an already introduced server
void wlinks_renameUser(const unsigned char* oldnick, const unsigned char* newnick);
  // process a rename event
void wlinks_removeUser(const unsigned char* nick, const unsigned char* reason);
  // process a rename event
  
// ##### -- send something to a user
void wlinks_sndMsgToUser(const unsigned char* nick,const unsigned char* msg);
  // send some IRC message to a user/to the server where this user is on (should be a valid IRC event message)






////        SECTION F - adding and removing channels

// ##### -- cycling through the known channels: --
#define CHANNELCYCLE_LOCAL 1
#define CHANNELCYCLE_ALL 2
#define CHANNELCYCLE_ALLOTHERS 3
//IMPORTANT! After returning control to WeIRCd and leaving the module code after processing something, the cycles are reset until the module gains control next time
void wlinks_cycleChannels_start(int cycletype);
  // start a new loop through all the channels matching the criteria specified by the type
int wlinks_cycleChannels_next();
  // get the handle for the next channel in the currently running channel cycling
void wlinks_cycleChannels_reset();
  // reset the channel cycling (not necessary in normal cases, just use wlinks_cycleUsers_start(type) to start a new one and it will automatically stop the old correctly)

// ##### -- get information on a channel handle --
const unsigned char* wlinks_channel_getName(int handle);
  // get the name of the channel represented by this channel handle
const unsigned char* wlinks_channel_getServer(int handle);
  // returns the server this channel is on. in case it is the local server, this returns NULL!
long long wlinks_channel_getTS(int handle);
  // returns the creation time of the channel as time stamp
int wlinks_channel_getHandle(const unsigned char* c);
  // returns a channel handle for the channel with this name or -1 when no such channel exists
void wlinks_introduceChannelUsers(int handle);
  // introduce the channel users of the channel represented by this handle to the current link by calling the module's send function with join events which then can be proceeded accordingly
void wlinks_introduceChannelUsersSilently(int handle);
  // same as command above, but sending SILENTJOIN event

// ##### -- add and remove channels and do things with them
int wlinks_collideChannel(const unsigned char* nickname, long long timestamp);
  // this function is going to deop all local users in this channel in case the local channel copy's time stamp is newer than the specified one and returns 0. Otherwise, it returns -1 and everyone stays opped (in this case, you should tell the other server to deop everyone in that chan)
void wlinks_sendTopic(const unsigned char* chan, const unsigned char* user);
  // send the topic of a chan to a user
void wlinks_sendNamesToUser(const unsigned char* chan, const unsigned char* user);
  // send the NAMES numerics to a user (only possible if the channel is local)
void wlinks_addChannel(const unsigned char* chan, const unsigned char* servername);
  // add a specific channel from an already introduced server
void wlinks_delChan(const unsigned char* chan);
  // delete this channel
bool wlinks_channelContainsUser(const unsigned char* chan, const unsigned char* user);
  // returns true if a specific user is on that chan (does only work if the channel or the user is local)
void wlinks_joinUser(const unsigned char* chan, const unsigned char* actor);
  // add a user into a channel WITHOUT checking for bans or passwords or anything (actor is the "nick!userid@host" form)
void wlinks_joinUserSilently(const unsigned char* chan, const unsigned char* actor);
  // same as before, but without sending JOIN event messages to local users
void wlinks_channelMsg(const unsigned char* chan, const unsigned char* msgline);
  // send a channel msg to all local users in that channel
void wlinks_partUser(const unsigned char* chan, const unsigned char* actor, const unsigned char* reason);
  // part a user from a channel (actor is the "nick!userid@host" form) with a displayed reason (mostly specified by the user himself/herself)
void wlinks_processJoin(const unsigned char* chan, const unsigned char* actor, const unsigned char* password);
  // try to have the user join the channel and check for bans, channel passwords etc (actor is the "nick!userid@host" form) and send corresponding join event message or error numeric back. Password should either be a password being specified by the joining user or NULL
  
